import {createRouter, createWebHashHistory} from "vue-router";

import route_vue_get from "../route/getVue.js";

import {createHead} from '@unhead/vue';

import {createI18n} from "vue-i18n";

import storage from "../storage.js";

import NProgress from "nprogress";

import alone from "../alone.js";

import safe from "../safe.js";

import {createApp} from "vue";

import app from "@/App.vue";

import apps from "./app.js";

const self = {
    //本地设置的路由,和远程路由
    route: {local: [], web: []},
    //加载本地配置
    local_conf() {
        const local = process.config || {data: null, random: null};
        const conf = ((local.data || '' && local.random || '') ? (safe.mov.de(local) || local) : local);
        if (alone.isset(conf)) {
            this.route.local = conf.route || [];
            apps.config.object(alone.del_key(conf, 'route'));
        }
        return this;
    },
    //加载远程配置
    async web_conf() {
        const conf_file = apps.get_env('conf');
        if (alone.isset(conf_file) && alone.is_string(conf_file)) {
            const res = await apps.http(conf_file).query({
                data: safe.url.en({
                    make: apps.get_make(),
                    origin: apps.get_origin(),
                    domain: alone.domain.domain()
                })
            }).get(function (res) {
                if ((res.data || '') && (res.random || '')) {
                    return (safe.mov.de(res) || res);
                } else if (res.code == 200) {
                    return res.data;
                } else if (alone.is_object(res)) {
                    return res;
                }
                return false;
            }, function () {
                return false;
            });
            if (alone.isset(res)) {
                this.route.web = res.route || [];
                apps.config.object(alone.del_key(res, 'route'));
            }
        }
        return this;
    },
    async process() {
        //有效时间
        const make_time = parseInt(apps.config.get('make_time', 28800));
        //生成访问标识
        if (!alone.isset(apps.get_make())) {
            storage.set('makeCache', alone.get_token(), make_time);
        }
        //保存第一次的来路
        if (!alone.isset(apps.get_origin())) {
            const origin = alone.origin.get();
            if (alone.isset(origin)) {
                storage.set('originCache', origin, make_time);
            }
        }
        this.local_conf();
        await this.web_conf();
        apps.set_title();
        return this;
    },
    //处理路由
    load_route(view = {}) {
        this.route_list = route_vue_get(Object.assign(apps.config.get('routing', {}) || {}, {view: view}), this.route.local || [], this.route.web || []);
        apps.config.list().route = this.route_list.route;
        this.route_config = this.route_list.routes || [];
        return this;
    },
    load_i18n() {
        this.i18n_config = {
            locale: apps.config.get('locale'),
            fallback: apps.config.get('fallback'),
            messages: apps.config.get('lang'),
        };
        return this;
    },
    start_route(array) {
        if (alone.isset(array)) {
            apps.routes = (alone.is_object(array) ? Object.values(array) : array);
            apps.route = createRouter({
                history: createWebHashHistory(import.meta.env.BASE_URL),
                routes: app.routes
            });
            const black = ' style="display: block"';
            const none = ' style="display: none"';
            const bar = (apps.config.get('load_bar') == true || apps.config.get('load_bar') == 1) ? black : none;
            const ico = (apps.config.get('load_ico') == true || apps.config.get('load_ico') == 1) ? black : none;
            //启动 进度条和加载图标
            const bar_start = () => NProgress.start();
            //关闭 进度条和加载图标
            const bar_close = () => NProgress.done();
            NProgress.settings.template = `<div class="bar" role="bar"${bar}><div class="peg"></div></div><div class="spinner" role="spinner"${ico}><div class="spinner-icon"></div></div>`;
            apps.route.beforeEach((to, from, next) => {
                const title = alone.get_arr(to, 'meta.title', null);
                if (alone.isset(title)) {
                    apps.set_title(title);
                }
                if (to.matched.length) {
                    bar_start();
                    next();
                } else {
                    const error = apps.config.get('error');
                    if (!error) {
                        return false;
                    }
                    next(error);
                }
            });
            apps.route.afterEach(() => (bar_close()));
            apps.route.onError(() => (bar_close()));
            create.use(apps.route);
            import('../style/css/bar.css');
        }
        return this;
    },
    start_i18n({locale, fallback, messages} = {}) {
        if (alone.isset(locale)) {
            locale = storage.get('langLocale') || locale;
            apps.i18n = createI18n({locale: locale, fallbackLocale: fallback || locale, messages: messages});
            apps.get_locale(locale, fallback);
            create.use(apps.i18n);
        }
        return this;
    },
    async main(view = {}) {
        await this.process();
        this.load_route(view).load_i18n();
        this.start_route(this.route_config).start_i18n(this.i18n_config);
        import('vfonts/Lato.css');
        import('vfonts/FiraCode.css');
        return this;
    }
};

export const create = createApp(app);

export const loadAloneFrame = async function () {
    await self.main();
}

export const mountApp = function () {
    create.mount('#' + apps.get_env('app'));
}

export const useMd = function () {
    create.use(createHead());
}

export default {app: app, create: create, createApp: createApp};